#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include "lcd.h"
#include "usart.h"
#include "i2c.h"
#include "MCP79410.h"

char buffer[64];
volatile uint8_t bufferIndex = 0;
volatile uint8_t newCommand = 0;
volatile uint16_t timeoutCounter = 0;
volatile uint8_t timeout = 0;

void getSeparatedNumbers(char str[], uint8_t outArray[], uint8_t outArraySize, char separator);
void adcInit(void);
void pwmInit(void);
void timeoutTimerInit(void);

int main(void) {
	i2cInit();
	lcdInit();
	usartInit();
	adcInit();
	pwmInit();
	timeoutTimerInit();
	sei();
	//wcz przerwanie
	UCSRB |= _BV(RXCIE);
	while (1) {
		if (newCommand) {
			newCommand = 0;
			if (strncmp(buffer, "ADC", 3) == 0) {
				ADCSRA |= _BV(ADSC);
				while(ADCSRA & _BV(ADSC));
				char str[16];
				sprintf(str, "U=%.3fV\r\n", ADCW * 5.0 / 1024);
				usartSendString(str);
			}
			if (strncmp(buffer, "TIMEGET", 7) == 0) {
				DateTime dateTime;
				rtcReadDateTime(&dateTime);
				char str[16];
				sprintf(str, "%02d:%02d:%02d\r\n", dateTime.hours, dateTime.minutes, dateTime.seconds);
				usartSendString(str);
			}
			if (strncmp(buffer, "TIMESET", 7) == 0) {
				DateTime newDateTime;
				uint8_t time[3];
				getSeparatedNumbers(buffer + 8, time, 3, ':');
				newDateTime.hours = time[0];
				newDateTime.minutes = time[1];
				newDateTime.seconds = time[2];
				rtcWriteDateTime(&newDateTime);
			}
			if (strncmp(buffer, "RGB", 3) == 0) {
				uint8_t rgb[3];
				getSeparatedNumbers(buffer + 4, rgb, 3, ' ');
				OCR1A = rgb[0];
				OCR1B = rgb[1];
				OCR0 = rgb[2];
			}
			if (strncmp(buffer, "LCDSTR", 6) == 0) {
				lcdString(buffer + 7);
			}
			if (strncmp(buffer, "LCDCLR", 6) == 0) {
				lcdWriteCommand(LCD_COMMAND_CLEAR);
			}
		}
		if (timeout) {
			timeout = 0;
			usartSendString("timeout\r\n");
		}
	}
}

void getSeparatedNumbers(char str[], uint8_t outArray[], uint8_t outArraySize, char separator) {
	uint8_t strPos = 0;
	uint8_t outPos = 0;
	for (uint8_t i = 0; i < outArraySize; i++) outArray[i] = 0;
	while ((str[strPos] != 0) && (outPos < outArraySize)) {
		if (str[strPos] == separator) {
			outPos++;
		} else {
			outArray[outPos] = outArray[outPos] * 10 + (str[strPos] - '0');
		}
		strPos++;
	}
}

void adcInit(void) {
	ADCSRA |= _BV(ADEN); //wcz ADC
	//AVCC jako napicie odniesienia, pomiar na pinie ADC3 (PA3)
	ADMUX |= _BV(REFS0) | _BV(MUX1) | _BV(MUX0);
	//preskaler 128
	ADCSRA |= _BV(ADPS0) | _BV(ADPS1) | _BV(ADPS2);
}

void pwmInit(void) {
	//Timer1 - PWM, 8 bitw, przeczanie OC1A i OC1B
	TCCR1A = _BV(WGM10) | _BV(COM1A1) |_BV(COM1B1);
	//bez preskalera
	TCCR1B = _BV(CS10);
	//Timer0 - PWM, przeczanie OC0
	TCCR0 = _BV(WGM00) | _BV(COM01) | _BV(CS01);
	DDRB |= _BV(DDB3); //OC0
	DDRD |= _BV(DDD5); //OC1A
	DDRD |= _BV(DDD4); //OC1B
}

void timeoutTimerInit(void) {
	//preskaler 1024, tryb CTC
	TCCR0 = _BV(CS02) | _BV(CS00) | _BV(WGM01);
	//reset timera co ok. 10 ms
	OCR0 = 155;
	//przerwanie dla CTC
	TIMSK |= _BV(OCIE0);
}

ISR(USART_RXC_vect) {
	uint8_t value = UDR;
	if (value != '\r') {
		if (bufferIndex < sizeof(buffer)) {
			buffer[bufferIndex++] = value;
		}
	} else {
		buffer[bufferIndex] = 0;
		bufferIndex = 0;
		newCommand = 1;
	}
	timeoutCounter = 0;
}

ISR(TIMER0_COMP_vect) {
	if (bufferIndex) {
		timeoutCounter++;
		if (timeoutCounter == 500) {
			timeoutCounter = 0;
			bufferIndex = 0;
			timeout = 1;
		}
	}
}